/**
 * @module ButtonGroup
 * @desc Depend.
 * <ul>
 *   <li>react</li>
 *   <li>bootstrap</li>
 *   <li>fontawesome</li>
 * </ul>
 * @param id
 * @param option
 * @param {array} option.data array
 * @param {string} option.value
 * @param {boolean} option.disabled
 * @param {object} option.className
 * @returns {*}
 */

'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'

export default class ButtonGroup extends React.Component {
  constructor(props) {
    super(props);

    let defaultClassName = _.clone(ButtonGroup.defaultProps.className);

    this.state = {
      data: this.props.data,
      value: this.props.value,
      disabled: this.props.disabled,
      className: _.extend(defaultClassName, this.props.className)
    };

    this.click = this.click.bind(this);
  }

  render() {
    return dom('div', {className: this.state.className.group, style: this.props.style, id: this.props.id},
      this.state.data.map((item, i) => {
        return dom('button', {
            className: this.className(item),
            disabled: this.disabled(),
            onClick: this.click,
            name: this.props.name,
            value: item.value,
            key: i,
            'data-keep': item.keep
          },
          item.icon ? dom('i', {className: item.icon}, item.title) : item.title
        )
      })
    );
  }

  componentWillReceiveProps(newProps) {
    if (!_.isEqual(this.state.data, newProps.data)) {
      this.setState({data: newProps.data});
    }

    if (this.state.value != newProps.value) {
      this.setState({value: newProps.value});
    }
  }

  className(item) {
    return this.state.className.base + ' ' + (
        this.state.value == item.value ? this.state.className.on : this.state.className.off
      );
  }

  disabled() {
    return this.state.disabled;
  }

  click(event) {
    event.preventDefault();

    //
    let keep = event.currentTarget.getAttribute('data-keep');
    if (!keep) {
      this.state.value = event.target.value;
      this.setState({value: this.state.value});
    }
    this.props.onClick(event);
    this.props.onChange(event);
  }

  /**
   * getValue - Gets the currently selected item value
   * @returns {*}
   */
  getValue() {
    return this.state.value;
  }

  /**
   * setValue - Sets the value
   * @param value
   */
  setValue(value) {
    this.setState({value: value});
  }

  /**
   * disable - Disable the button, true: disable false: enable
   * @param disabled
   */
  setDisable(disabled) {
    this.setState({disabled: disabled});
  }
}

ButtonGroup.defaultProps = {
  data: [],
  className: {
    base: 'btn',
    group: 'btn-group',
    on: 'btn-info',
    off: 'btn-default'
  },
  onChange: () => {
  },
  onClick: () => {
  }
};

ButtonGroup.init = (id, option) => {
  return ReactDOM.render(
    React.createElement(ButtonGroup, option),
    document.getElementById(id)
  );
};
