/**
 * @module ComboBox
 * @desc depend.
 * <ul>
 *   <li>react</li>
 *   <li>bootstrap</li>
 *   <li>select2</li>
 * </ul>
 *
 * https://select2.github.io/options.html
 *
 * @param id
 * @param option
 * @returns {*}
 */

'use strict';

import React      from 'react'
import ReactDOM   from 'react-dom'
import * as net   from '../core/net'
import * as util  from '../core/util'

export default class MultiSelect extends React.Component {

  constructor(props) {
    super(props);

    this.state = {
      multiple: this.props.multiple,
      data: this.props.data
    };
  }

  render() {

    const option = {
      multiple: this.state.multiple,
      style: this.props.style,
      id: this.props.id,
      name: this.props.name,
      disabled: this.props.disabled,
      className: this.props.className
    };

    // 拷贝 data- 开头的属性
    Object.keys(this.props).filter(key => {
      return key.startsWith('data-');
    }).forEach(key => option[key] = this.props[key]);

    return dom('select', option,
      dom('option', ) // The first blank item
    );
  }

  componentDidUpdate(prevProps, prevState) {

    // If the data list has changed, re-create the component
    if (!_.isEqual(prevState.data, this.state.data)) {
      this.createDataNode(true);
    }

    if (!_.isEqual(prevProps.data, this.props.data)) {
      this.setState({data: this.props.data});
    }

    if (this.props.api !== prevProps.api ||
      this.props.dataUk !== prevProps.dataUk ||
      this.props.dataName !== prevProps.dataName) {
      this.fetch();
    }

    if (this.props.multiple !== prevProps.multiple) {
      this.setState({multiple: this.props.multiple});
    }

    // If the value has changed, then the reaction to the component
    if (this.props.value !== this.node.val()) {
      if (typeof this.props.value !== 'undefined') {
        this.node.val(this.props.value);
        this.node.trigger('change.select2');
      }
    }
  }

  componentWillUnmount() {
    this.node.select2('destroy').empty();
  }

  componentDidMount() {
    this.node = $(ReactDOM.findDOMNode(this));
    this.node.on('change', this.props.onChange);

    // 如果有没有设定的参数，则不进行网络请求
    if (this.props.api && !this.hasUnsetParams()) {
      // If the API is specified, then a network request is sent
      this.fetch();
    } else {
      // If the Data is specified.
      this.createDataNode();
    }
  }

  hasUnsetParams() {
    return Object.values(util.params(this.props.api)).find(item => item.startsWith('@'));
  }

  fetch() {
    net.get(this.props.api.split('?')[0], {condition: util.params(this.props.api)}, (err, result) => {
      if (err) {
        return console.log(err);
      }

      if (result.items.length <= 0) {
        this.setState({data: {id: '', text: ''}});
      }

      this.setState({
          data: result.items.map(
            item => Object.assign({id: item[this.props.dataUk], text: item[this.props.dataName]})
          )
        }
      );
    });
  }

  createDataNode(destroy) {

    const option = {
      theme: this.props.theme,
      placeholder: this.props.placeholder,
      minimumResultsForSearch: this.props.search ? 1 : -1,
      data: this.state.data
    };

    if (destroy && this.node.data('select2')) {
      this.node.select2('destroy').empty().select2(option);
    } else {
      this.node.select2(option);
    }

    if (!_.isUndefined(this.props.value)) {
      this.node.val(this.props.value);
      this.node.trigger('change.select2');
    }
  }

  // API模式时，只有点击是才出发获取后台数据。如果要显示缺省被选中的值，因为还没有备选数据所以没有办法显示，固暂时不使用
  // createApiNode(destroy) {
  //
  //   const option = {
  //     theme: this.props.theme,
  //     placeholder: this.props.placeholder,
  //     minimumResultsForSearch: this.props.search ? 1 : -1,
  //     ajax: {
  //       url: this.props.api.split('?')[0],
  //       dataType: 'json',
  //       data: params => {
  //         return {
  //           skip: ((params.page || 1) - 1) * this.props.pageSize,
  //           limit: this.props.pageSize,
  //           condition: Object.assign({keyword: params.term}, util.params(this.props.api))
  //         };
  //       },
  //       processResults: (result, params) => {
  //         const page = params.page || 1,
  //           data = result.data.items.map(
  //             item => Object.assign({id: item[this.props.dataUk], text: item[this.props.dataName]})
  //           );
  //         return {
  //           results: data,
  //           pagination: {more: (page * this.props.pageSize) < result.data.totalItems}
  //         };
  //       },
  //       cache: true
  //     }
  //   };
  //
  //   if (destroy) {
  //     return this.node.select2('destroy').empty().select2(option);
  //   }
  //   this.node.select2(option);
  // }

  getValue() {
    if (Array.isArray(this.node.val())) {
      return this.node.val().filter(item => item !== '');
    }
    return this.node.val();
  }

  setValue(value) {
    this.node.val(value);
  }

  value(data) {
    if (data) {
      this.node.val(data);
    }
    return this.node.val();
  }
}

MultiSelect.defaultProps = {
  dataUk: '_id',
  dataName: 'name',
  dataKeys: 'keys',
  disabled: false,
  theme: 'classic',
  search: true,           // 是否显示检索栏
  condition: {},
  style: {width: '100%'},
  multiple: '',
  placeholder: '',
  pageSize: 25,           // 一页显示的项目数（指定api时有效）
  onChange: () => {
  }
};

MultiSelect.init = (id, option) => {
  return ReactDOM.render(
    React.createElement(MultiSelect, option),
    document.getElementById(id)
  );
};
