/**
 * @module Mask
 * @param id
 * @param option
 * @param {number} option.timeout - Mask The maximum time to display (second). default: 60 sec.
 * @returns {*}
 */

'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'

export default class Mask extends React.Component {
  constructor(props) {
    super(props);
    this.state = {counter: 0}
  }

  componentDidMount() {
    $(document)
      .bind("ajaxSend", function () {
        this.show();
      }.bind(this))
      .bind("ajaxComplete", function () {
        this.hide();
      }.bind(this));
  }

  render() {
    return dom('div', {
      style: {
        position: 'fixed',
        top: '0px',
        left: '0px',
        width: '100%',
        height: '100%',
        zIndex: '1000',
        backgroundColor: '#000',
        opacity: '0.2',
        filter: 'alpha(opacity=20)',
        display: this.state.counter > 0 ? 'block' : 'none'
      }
    });
  }

  /**
   * show mask
   */
  show() {
    this.state.counter = this.state.counter + 1;
    this.setState({counter: this.state.counter});

    clearTimeout(this.timer);
    this.timer = setTimeout(function () {
      this.hide();
    }.bind(this), this.props.timeout * 1000);
  }

  /**
   * hide mask
   */
  hide() {
    if (this.state.counter < 1) {
      clearTimeout(this.timer);
    }

    this.setState({counter: this.state.counter - 1});
  }
}

Mask.init = function (option) {

  $('body').prepend('<div id="mask-element"></div>');

  return ReactDOM.render(
    React.createElement(Mask, option),
    document.getElementById('mask-element')
  );
};

Mask.defaultProps = {
  timeout: 60 * 1000
};
