/**
 * @module alert.js
 * @desc Depend.
 * <ul>
 *   <li>toastr</li>
 *   <li>alertify</li>
 * </ul>
 *
 * @see https://github.com/MohammadYounes/AlertifyJS
 * @see https://github.com/t4t5/sweetalert
 *
 */

'use strict';

import {i} from './util'

export function success(title, message) {
  alertify.success(title, message);
  console.info(title, message);
}

export function info(title, message) {
  alertify.success(title, message);
  console.info(title, message);
}

export function error(message, option) {

  if (option) {
    console.error(message, option);
  } else {
    console.error(message);
  }

  // 字符串参数原样显示
  if (typeof message === 'string') {
    return alertify.error(message);
  }

  // 对象类型，尝试解析内容
  if (typeof message === 'object') {

    // 详细信息
    if (message.errors && message.errors.length > 0) {
      return alertify.error(message.errors.map(item => item.message).join('<br>'));
    }

    // 概要信息
    if (message.message) {
      return alertify.error(message.message);
    }

    // 都没有显示option
    if (option) {
      return alertify.error(option);
    }

    // 还是没有，显示原生错误信息
    if (message.statusText) {
      return alertify.error(message.statusText);
    }

    alertify.error('系统异常，请稍后再试');
  }
}

export function warn(title, message) {
  alertify.warning(title, message);
  console.warn(title, message);
}

export function confirm(title, message, callback) {

  if (typeof message === 'function') {
    callback = message;
    message = title;
    title = 'Confirm';
  }

  alertify.confirm(title, message,
    () => {
      callback(true);
    },
    () => {
      callback(false);
    }).set('labels', {ok:i('common.ok'), cancel:i('common.cancel')});
}

export function prompt(title, message, value, callback) {

  if (typeof value === 'function') {
    callback = value;
    value = '';
  }

  if (typeof message === 'function') {
    callback = message;
    message = title;
    title = 'Prompt';
    value = '';
  }

  if (typeof title === 'function') {
    callback = title;
    message = '';
    title = 'Prompt';
    value = '';
  }

  alertify.prompt(title, message, value,
    (evt, value) => {
      callback(true, value);
    },
    () => {
      callback(false);
    });
}

/**
 * Alertify setting
 *  ex.
 *  - The message box displays the time
 *    light.alert.set('notifier','delay', 6);
 *  - The message box displays the location
 *    light.alert.set('notifier','position', 'top-right');
 * @param args
 */
export function set(...args) {
  alertify.set(...args);
}
