/**
 * validation.js
 *
 * Validation是一个普通的Class，并非React组件
 * see. https://jqueryvalidation.org/documentation/
 */

'use strict';

import {i} from './util'

export default class Validation {

  constructor(form, option) {

    Validation.message();

    this.node = form instanceof HTMLElement ? $(form) : $('#' + form);

    if (option.errorStyle === 'label') {
      option.errorPlacement = function (error, element) {
        $(element)
          .closest('form')
          .find('label[for="' + element.attr("id") + '"]')
          .append(error);
      };
    }

    // 想让validator支持combobox组件，有些问题暂时注释掉
    // option.highlight = function (element, errorClass, validClass) {
    //   let node = $(element);
    //   if (node.hasClass('select2-hidden-accessible')) {
    //     node.next().find('.select2-selection').addClass('form-control ' + errorClass);
    //   } else {
    //     node.addClass(errorClass);
    //   }
    // };

    this.component = this.node.validate(option);
  }

  valid() {
    return this.node.valid();
  }

  rules() {
    // TODO:
  }
}


Validation.message = () => {
  $.extend($.validator.messages, {
    required: i('common.This field is required.'),
    remote: i('common.Please fix this field.'),
    email: i('common.Please enter a valid email address.'),
    url: i('common.Please enter a valid URL.'),
    date: i('common.Please enter a valid date.'),
    dateISO: i('common.Please enter a valid date (ISO).'),
    number: i('common.Please enter a valid number.'),
    digits: i('common.Please enter only digits.'),
    creditcard: i('common.Please enter a valid credit card number.'),
    equalTo: i('common.Please enter the same value again.'),
    accept: i('common.Please enter a value with a valid extension.'),
    // maxlength: $.validator.format('Please enter no more than {0} characters.'),
    // minlength: $.validator.format('Please enter at least {0} characters.'),
    // rangelength: $.validator.format('Please enter a value between {0} and {1} characters long.'),
    // range: $.validator.format('Please enter a value between {0} and {1}.'),
    // max: $.validator.format('Please enter a value less than or equal to {0}.'),
    // min: $.validator.format('Please enter a value greater than or equal to {0}.')
  });
};

Validation.init = (form, option) => {

  option = option || {};

  // 错误消息组件的名称
  option.errorElement = option.errorElement || 'span';

  // 显示错误的位置 bottom - 在控件底部显示 | label - 在控件上面的label组件旁边显示
  option.errorStyle = option.errorStyle || 'label';

  return new Validation(form, option);
};
