'use strict';

const Webpack = require('webpack')
  , Extract = require('extract-text-webpack-plugin');

module.exports = {
  entry: {
    light: './index.js',
    css: [
      '../../css/light.less'
    ]
  },

  output: {
    path: __dirname,
    filename: '../../sdk/[name].js',
    library: 'light',
    libraryTarget: 'umd'
  },

  module: {
    rules: [
      {
        test: /\.js$/,
        exclude: /(node_modules)/,
        use: {
          loader: 'babel-loader',
          options: {
            presets: ['env']
          }
        }
      },
      {
        test: /\.less$/,
        use: Extract.extract(
          {
            fallback: 'style-loader',
            use: ["css-loader?minimize=true", "less-loader"]
          }
        )
      }
    ]
  },

  externals: {
    'jquery': 'jQuery',
    '_': 'lodash',
    'react': 'React',
    'react-dom': 'ReactDOM',
    'alertifyjs': 'alertify'
  },

  plugins: [

    // 防止webpack对nodejs模块进行打包 （light的net模块会使用request）
    // new Webpack.IgnorePlugin(
    //   new RegExp("^(request)$")
    // ),

    // 压缩
    // new Webpack.optimize.UglifyJsPlugin(
    //   {
    //     compress: {
    //       warnings: false
    //     }
    //   }
    // ),

    new Webpack.ProvidePlugin({
      'dom': ['react', 'createElement']
    }),

    new Extract(
      {
        filename: '../../sdk/light.css',
        allChunks: true
      }
    )
  ]
};
