/**
 * @file 表单头
 * @module form-header
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

import React      from 'react'
import CONST      from '../constant'
import Plugin     from './plugin'

export default class Header extends React.Component {

  constructor(props) {
    super(props);

    this.state = {closed: this.props.closed, title: this.props.title, plugin: this.props.plugin};
    this.onExpanderClick = this.onExpanderClick.bind(this);
    this.onFormClick = this.onFormClick.bind(this);
    this.onPluginClick = this.onPluginClick.bind(this);
  }


  render() {
    return dom('header', {onClick: this.onFormClick},
      this.state.title,
      this.expander(),
      this.plugin()
    );
  }


  plugin() {
    if (typeof this.state.plugin === 'undefined') {
      return null;
    }

    if (Plugin.list('all').length <= 0) {
      return null;
    }

    return dom('a', {href: '#', className: 'plugin-handler', onClick: this.onPluginClick},
      dom('i', {className: this.state.plugin ? 'fa fa-th' : 'fa fa-plug'})
    );
  }


  expander() {
    if (typeof this.state.closed === 'undefined') {
      return null;
    }

    return dom('a', {href: '#', style: {float: 'right'}, onClick: this.onExpanderClick},
      dom('i', {className: this.state.closed ? 'fa fa-angle-up' : 'fa fa-angle-down'})
    );
  }


  componentWillReceiveProps(nextProps) {
    if (this.state.title !== nextProps.title) {
      this.setState({title: nextProps.title});
    }
  }


  onExpanderClick(event) {
    event.preventDefault();

    const closed = !this.state.closed;

    this.setState({closed: closed});
    this.props.emit(CONST.EVENT.FOLDFORM, closed);
  }


  onPluginClick(event) {
    event.preventDefault();

    const plugin = !this.state.plugin;

    this.setState({plugin: plugin});
    this.props.emit(CONST.EVENT.PLUGIN, plugin);
  }


  onFormClick(event) {
    event.preventDefault();

    this.props.emit(CONST.EVENT.SELECTFORM);
  }
}

Header.defaultProps = {
  title: '',
  closed: undefined,
  plugin: undefined,
  emit: () => {
  }
};
