/**
 * @file 表单
 * @module define.form
 * @author r2space@gmail.com
 * @version 1.0.0
 */

'use strict';

import React          from 'react'
import ReactDOM       from 'react-dom'
import CONST          from '../constant'
import Widget         from '../widget/widget'
import Header         from './form-header'
import Footer         from './form-footer'
import FieldSet       from './form-fieldset'
import i              from './localize'


/**
 * @desc 表单
 */
export default class Panel extends React.Component {

  constructor(props) {
    super(props);

    this.meta = this.props.meta;
    this.emitter = this.props.emitter || $({});

    this.emit = this.emit.bind(this);
    this.onAdd = this.onAdd.bind(this);
    this.onRemove = this.onRemove.bind(this);
    this.onMove = this.onMove.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onSelect = this.onSelect.bind(this);
    this.onExpand = this.onExpand.bind(this);
    this.onAddRow = this.onAddRow.bind(this);
    this.onRemoveRow = this.onRemoveRow.bind(this);
    this.onCopy = this.onCopy.bind(this);
    this.onSelectForm = this.onSelectForm.bind(this);
    this.onFoldBlock = this.onFoldBlock.bind(this);

    this.emitter.on(CONST.EVENT.SELECT, this.onSelect);         // 选择组件
    this.emitter.on(CONST.EVENT.EXPAND, this.onExpand);         // 扩大组件
    this.emitter.on(CONST.EVENT.ADD, this.onAdd);               // 添加组件
    this.emitter.on(CONST.EVENT.REMOVE, this.onRemove);         // 删除组件
    this.emitter.on(CONST.EVENT.MOVE, this.onMove);             // 移动组件
    this.emitter.on(CONST.EVENT.CHANGE, this.onChange);         // 组件值发生变化
    this.emitter.on(CONST.EVENT.SELECTFORM, this.onSelectForm); // 选择表单
    this.emitter.on(CONST.EVENT.ADDROW, this.onAddRow);
    this.emitter.on(CONST.EVENT.REMOVEROW, this.onRemoveRow);
    this.emitter.on(CONST.EVENT.COPY, this.onCopy);
    this.emitter.on(CONST.EVENT.FOLDBLOCK, this.onFoldBlock);   // 折叠Block
    this.emitter.on(CONST.EVENT.FOLDFORM, this.onFold);         // 折叠Form

    this.state = {
      data: this.props.data,                                    // 表单组件数据
      option: {
        closed: undefined,                                      // 表单状态: 表单是否可折叠, undefined为不显示折叠按钮, false为关闭状态
        selected: [],                                           // 表单状态: 表单单元格选中状态, [row, col]
        scroll: this.props.scroll,                              // 表单状态: 是否显示滚动条
        hidden: [],                                             // 表单状态: 隐藏的组件（Block折叠等操作时，保存隐藏的组件 row, col。用途不同所以未使用组件本身的hidden）

        mode: this.props.mode || CONST.MODE.DESIGN,             // view 视图模式, readonly 只读模式, input 编辑模式, design 设计模式
        button: this.props.button,                              // 操作按钮

        h: this.meta.row || CONST.Config.Height,                // 默认的form行数
        w: this.meta.column || CONST.Config.Width,              // 默认的form列数
        title: this.meta.name || i(this.props.title),           // 标题
      }
    };
  }

  render() {

    // panel与子元素间，传递三个参数分别是 data, option, emit
    return React.createElement('form', {className: 'light-form'},
      React.createElement(Header, {
          title: this.state.option.title, closed: this.state.option.closed, emit: this.emit
        }
      ),
      React.createElement(FieldSet, {
          data: this.state.data,
          option: Object.assign({}, this.state.option, {form: this.state.data, emitter: this.emitter}),
          emit: this.emit,
          style: this.props.style
        }
      ),
      React.createElement(Footer, {button: this.state.option.button}),
      // 添加一个不可见输入框, 防止只有一个输入框时敲回车提交表单
      React.createElement('input', {type: 'text', style: {display: 'none'}})
    );
  }

  emit(type, data, node) {
    this.emitter.trigger(type, [data, node]);
  }

  onFold(event, state) {
    this.state.option.closed = state;
    this.setState({option: this.state.option});
  }

  onChange(event, data) {
    let node = _.find(this.state.data, function (item) {
      return item.col === data.col && item.row === data.row;
    });
    node.value = data.value;

    this.props.onChange(data, this.state.data);
    this.setState({data: this.state.data});
  }

  onSelect(event, data) {
    this.state.option.selected = [data.row, data.col];
    this.setState({option: this.state.option});
  }

  onSelectForm() {
    this.state.option.selected = [];
    this.setState({option: this.state.option});
  }

  onFoldBlock(event, data) {

    let node = _.find(this.state.data, function (item) {
      return item.col === data.col && item.row === data.row;
    });

    node.closed = data.closed;

    this.buildHiddenBlock();
    this.setState({option: this.state.option});
  }

  onAdd(event, data) {
    let node = _.find(this.state.data, function (item) {
      return item.col === data.col && item.row === data.row;
    });

    if (!node) {
      node = {col: data.col, row: data.row, colSpan: 1, rowSpan: 1};
      this.state.data.push(node);
    }

    node.type = data.type;
    node.title = _.isUndefined(data.title) ? data.type : data.title;
    node.name = Widget.guid();

    this.autoAppendRow();
    this.buildPath();
    this.props.onWidgetChange(data, this.state.data);
    this.setState({data: this.state.data});
    this.emit(CONST.EVENT.SELECT, node);
  }

  onCopy(event, data) {

    // 添加一行（后续组件行数位置加一）
    this.state.data.forEach(item => {
      if (item.row > data.row) {
        item.row = item.row + 1;
      }
    });

    // Deep clone
    data = Widget.clone(data);
    data.row = data.row + 1;
    data.name = Widget.guid();
    this.state.data.push(data);

    this.buildPath();
    this.props.onWidgetChange(data, this.state.data);
    this.setState({data: this.state.data});
  }

  onRemove(event, data) {
    this.state.data = _.reject(this.state.data, function (item) {
      return data.col === item.col && data.row === item.row;
    });

    for (let i = data.col; i < data.col + data.colSpan; i++) {
      this.state.data.push({row: data.row, rowSpan: data.rowSpan, col: i, colSpan: 1});
    }

    this.state.option.selected = [];

    this.buildPath();
    this.props.onWidgetChange(data, this.state.data);
    this.setState({data: this.state.data, option: this.state.option});
    this.emit(CONST.EVENT.SELECT, {});
  }

  onMove(event, data) {

    // 被移走了组件的行, 后续组件位置加1
    _.each(this.state.data, item => {
      if (item.row === data.source.row && item.col > data.source.col) {
        item.col = item.col + data.source.colSpan - 1;
      }
    });

    // 目的地位置上有空组件存在（添加行等操作存在的占位item），则先删除
    this.state.data = _.reject(this.state.data, item => {
      return item.col === data.col && item.row === data.row;
    });

    let node = _.find(this.state.data, item => {
      return item.col === data.source.col && item.row === data.source.row;
    });

    node.row = data.row;
    node.col = data.col;
    node.rowSpan = data.source.rowSpan;
    node.colSpan = data.source.colSpan;

    // 移动位置上有组件在，则缩小组件的宽度
    for (let index = data.col + 1; index < data.col + data.source.colSpan; index++) {
      let blocker = _.find(this.state.data, function (item) {
        return item.col === index && item.row === data.row;
      });

      if (blocker && blocker.type) {
        node.colSpan = index - data.col;
        break;
      }
    }

    // 如果超出了表单宽度, 则缩小组件宽度
    if (node.col + node.colSpan > this.state.option.w) {
      node.colSpan = this.state.option.w - node.col;
    }

    // 如果设定了span, 所以后续组件位置减1
    if (node.colSpan > 1) {
      this.state.data.forEach(item => {
        if (item.row === data.row && item.col > data.col) {
          item.col = item.col - node.colSpan + 1;
        }
      });
    }

    this.state.option.selected = [];

    this.autoAppendRow();
    this.buildPath();
    this.buildHiddenBlock();
    this.props.onWidgetChange(data, this.state.data);
    this.setState({data: this.state.data, option: this.state.option});
    this.emit(CONST.EVENT.SELECT, {});
  }

  onExpand(event, data) {

    // 修改组件的宽度
    let node = _.find(this.state.data, function (item) {
      return item.col === data.col && item.row === data.row;
    });

    node.colSpan = node.colSpan + 1;

    // 设定了span + 1, 所以后续组件位置减1
    this.state.data.forEach(item => {
      if (item.row === data.row && item.col > data.col) {
        item.col = item.col - 1;
      }
    });

    this.setState({data: this.state.data});
  }

  onAddRow(event, data) {
    this.state.data.forEach(item => {
      if (item.row > data.row) {
        item.row = item.row + 1;
      }
    });

    this.state.data.push({row: data.row + 1, col: 0, rowSpan: 1, colSpan: 1});

    this.buildPath();
    this.props.onWidgetChange(data, this.state.data);
    this.setState({data: this.state.data});
  }

  onRemoveRow(event, data) {
    this.state.data = _.reject(this.state.data, function (item) {
      let remove = (data.row === item.row);

      if (item.row > data.row) {
        item.row = item.row - 1;
      }

      return remove;
    });

    this.state.option.selected = [];

    this.buildPath();
    this.buildHiddenBlock();
    this.props.onWidgetChange(data, this.state.data);
    this.setState({data: this.state.data, option: this.state.option});
    this.emit(CONST.EVENT.SELECT, {});
  }

  autoAppendRow() {

    // 有控件的最大Section行数
    let max = _.maxBy(this.state.data, item => {
      return item.type ? item.row : 0;
    }).row;

    // 最大Section行数
    let total = _.maxBy(this.state.data, item => {
      return item.row;
    }).row;

    total = Math.max(total, this.state.option.h - 1);

    // 剩余行数不足3行，添加到3行
    let residue = total - max;
    if (residue < 3) {
      this.state.data.push({row: total + 3 - residue, rowSpan: 1, col: 0, colSpan: 1});
    }
  }

  buildPath() {

    // 排序
    this.state.data = _.sortBy(this.state.data, item => {
      return Number(`${item.row}.${item.col}`);
    });

    // 遍历所有的组件，根据所属的Block，SubBlock为每个组件设定path（目前只支持一层嵌套 Block > SubBlock > Item）
    let top = null, second = null, scope = 0;
    this.state.data.forEach(item => {

      if (!item.type) {
        return;
      }

      if (this.isBlock(item.type)) {

        top = item.name;
        second = null;
        scope = item.scope ? item.row + item.scope : 0;
        return;
      }

      if (this.isSubBlock(item.type)) {

        second = item.name;
        item.path = _.compact([top]);
        return;
      }

      // 如果block包含scope属性，那么超过scope指定的行时，视为scope外不设定path值
      if (scope && item.row > scope) {
        item.path = [];
        return;
      }

      item.path = _.compact([top, second]);
    });
  }

  buildHiddenBlock() {

    let hide = [];

    this.state.data.forEach(item => {

      if (!item.type || !this.isBlock(item.type)) {
        return;
      }

      let start = item.row + 1;
      let end = item.scope ? start + item.scope : 0;

      if (!end) {
        // 获取下一个Block组件的行数
        end = _.find(this.state.data, item => {
          return item.row >= start && this.isBlock(item.type);
        });

        if (end) {
          end = end.row;
        } else {

          // 如果没有找到，获取最后一个元素的行数
          end = _.maxBy(this.state.data, item => item.type ? item.row : 0).row + 1;
        }
      }

      // 如果分组被折叠起来，那么他下面的行设为隐藏
      if (item.closed) {
        hide = _.union(hide, _.range(start, end));
      }
    });

    this.state.option.hidden = hide;
  }

  isBlock(type) {

    if (!type) {
      return false;
    }

    if (type === CONST.WIDGET_TYPE_BLOCK) {
      return true;
    }

    let clazz = Widget.defaultProps.control[type];

    // 是否是继承自Block
    return (Object.getPrototypeOf(clazz).name === CONST.WIDGET_TYPE_BLOCK);
  }

  isSubBlock(type) {

    if (!type) {
      return false;
    }

    if (type === CONST.WIDGET_TYPE_SUBBLOCK) {
      return true;
    }

    let clazz = Widget.defaultProps.control[type];

    // 是否是继承自SubBlock
    return (Object.getPrototypeOf(clazz).name === CONST.WIDGET_TYPE_SUBBLOCK);
  }

  setWidth(width) {

    // 删除宽度以外都组件
    this.state.data = _.reject(this.state.data, item => {
      return item.col >= width;
    });

    // span超过宽度, 则缩小组件
    this.state.data.forEach(item => {
      if (item.col + item.colSpan > width) {
        item.colSpan = width - item.col;
      }
    });

    this.state.option.w = width;
    this.setState({option: this.state.option});
  }

  /**
   * 修改表单信息
   * @param source
   * @param target
   */
  setFormInfo(source, target) {

    // 通常该方法在属性变化时调用。开发者手动调用时，可以只指定target来改变表单信息
    if (_.isUndefined(target)) {
      target = source;
    }

    this.props.onFormInfoChange(source, target, this.meta);

    // 修改表单列数
    if (target.column) {
      this.setWidth(target.column);
    }

    // 修改表单名称
    if (target.name) {
      this.state.option.title = target.name;
      this.setState({option: this.state.option});
    }

    // 其他表单信息
    Object.assign(this.meta, target);
  }

  setWidget(source, target) {

    // 设定组件信息 属性窗口调用
    _.extend(_.find(this.state.data, item => {
      return target.col === item.col && target.row === item.row;
    }), target);

    // scope属性发生变化时，重新设置Path
    if (target.type === CONST.WIDGET_TYPE_BLOCK && source.name === 'scope') {
      this.buildPath();
    }

    this.props.onWidgetPropertiesChange(source, target, this.state.data);
    this.setState({data: this.state.data});
  }

  setAllWidget(data) {
    this.setState({data: data || []});
  }

  getFormInfo() {

    this.meta.public = _.isUndefined(this.meta.public) ? false : this.meta.public;
    this.meta.id = this.meta.id || Widget.guid();
    this.meta.active = true;
    this.meta.ver = this.meta.ver || '1.0.0';
    this.meta.items = _.filter(this.state.data, item => {
      return item.type;
    });

    delete this.meta._id;
    return this.meta;
  }

  getFormValue() {

    let result = {};

    this.state.data.forEach(item => {
      if (!item.type || !item.name) {
        return;
      }

      if (item.type === CONST.WIDGET_TYPE_BLOCK || item.type === CONST.WIDGET_TYPE_SUBBLOCK) {
        return;
      }

      let value = _.isUndefined(item.value) ? item.defaults : item.value;
      if (_.isUndefined(value)) {
        return;
      }

      result[item.name] = value;
    });

    return result;
  }

  selectForm() {
    this.emit(CONST.EVENT.SELECTFORM);
  }

  selectWidget(node) {
    this.emit(CONST.EVENT.SELECT, node);
  }
}

Panel.versionUp = (now) => {
  if (!now) {
    return '1.0.0';
  }

  let v = String(parseInt(now.replace(/\./g, '')) + 1);

  let patch = v.substr(-1)
    , minor = v.substr(-2)[0]
    , major = v.substr(0, v.length - 2);

  return `${major}.${minor}.${patch}`;
};

Panel.defaultProps = {
  title: 'body.form.defaultTitle',                          // 默认表单标题
  meta: {                                                   // 表基本信息
    row: CONST.Config.Height,
    column: CONST.Config.Width
  },
  data: [],
  scroll: false,
  style: {                                                  // 支持可扩展的 css style
    section: {}                                             // 编辑画面单元格 style
  },
  emitter: undefined,                                       // 每个Form需要有一个自己的事件对象，以便在多个Form同时存在时不互相影响
  onChange: (source, widgets) => {                          // 表单元素值发生变化
  },
  onWidgetChange: (source, widgets) => {                    // 表单元素发生变化(添加，移动等)
  },
  onWidgetPropertiesChange: (source, target, widgets) => {  // 表单元素属性发生变化
  },
  onFormInfoChange: (source, target, meta) => {             // 表单信息发生变化
  }
};

Panel.init = (node, option) => {
  return ReactDOM.render(
    React.createElement(Panel, option),
    document.getElementById(node)
  );
};
