/**
 * plugin.js
 */

'use strict';

import React    from 'react'
import CONST    from '../constant'

export default class Plugin extends React.Component {

  constructor(props) {
    super(props);
    this.onItemClick = this.onItemClick.bind(this);
  }

  /**
   * 获取有效的插件
   * - 插件如果指定了租户，那么租户需要匹配
   * - 如果参数type指定了，那么只返回type匹配的插件
   * @param type
   * @returns {Array}
   */
  static list(type) {
    if (typeof plugin === 'undefined') {
      return [];
    }

    // Current tenant
    const tenant = light.util.code();

    return plugin.filter(item => {
      if (!item.defaultProps) {
        console.log('The plugin default props is not defined.');
        return false;
      }

      if (!item.defaultProps.id) {
        console.log('The plugin id is not defined.');
        return false;
      }

      const scope = item.defaultProps.scope, code = item.defaultProps.code;
      if (!scope) {
        console.log('The plugin scope is not defined.');
        return false;
      }

      // 如果指定了租户，那么租户需要匹配
      if (code && tenant) {
        if ((Array.isArray(code) ? code : [code]).indexOf(tenant) < 0) {
          return false;
        }
      }

      if (type === 'all') {
        return true;
      }

      return (Array.isArray(scope) ? scope : [scope]).indexOf(type) >= 0;
    });
  }

  render() {

    let items = Plugin.list(this.props.type);
    if (items.length <= 0) {
      return dom('div', {style: {padding: '25px'}}, 'NO DATE');
    }

    // 名称排序
    items.sort((a, b) => {
      return a.defaultProps.id > b.defaultProps.id;
    });

    return dom('div', {className: 'list-group plugin-panel'},
      items.map(item => {

        const extend = this.props.data.extend || {}
          , id = item.defaultProps.id
          , state = extend.hasOwnProperty(id) ? 'on' : 'off';

        return dom('a', {
            'data-id': id,
            'data-state': state,
            key: id,
            className: 'list-group-item',
            onClick: this.onItemClick
          },
          dom('i', {
            className: `fa fa-toggle-${state}`,
            style: {float: 'right', fontSize: '24px', marginTop: '-2px'}
          }),
          dom('h4', {className: 'list-group-item-heading'}, item.defaultProps.id),
          dom('p', {className: 'list-group-item-text'}, item.defaultProps.description)
        )
      })
    );
  }

  onItemClick(event) {

    const id = event.currentTarget.getAttribute('data-id')
      , state = event.currentTarget.getAttribute('data-state');

    let extend = this.props.data.extend || {};
    if (state === 'off') {
      if (!extend.hasOwnProperty(id)) {
        extend[id] = null;
      }
    } else {
      delete extend[id];
    }

    this.props.emit(CONST.EVENT.CHANGE, {name: 'extend', value: extend});
  }
}