/**
 * @file
 * @module widget-block.js
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'
import CONST    from '../constant'

export default class Block extends Widget {

  constructor(props) {
    super(props);
    this.state = {closed: this.props.data.closed};
    this.onFoldClick = this.onFoldClick.bind(this);

    if (_.isUndefined(this.props.data.description)) {
      this.props.data.description = this.props.description;
    }
  }

  render() {

    return dom('div', {className: 'block'},
      dom('i', {className: 'fa fa-fw fa-' + this.className()}),
      dom('a', {href: '#', onClick: this.onFoldClick},
        dom('i', {className: 'fa ' + (this.state.closed ? 'fa-angle-up' : 'fa-angle-down')})
      ),
      dom('div', {}, this.description())
    );
  }

  description() {
    return this.props.data.description;
  }

  className() {
    return this.props.data.blockIcon || 'th-large';
  }

  onFoldClick(event) {

    event.preventDefault();

    let closed = !this.state.closed;
    this.setState({closed: closed});
    this.props.emit(CONST.EVENT.FOLDBLOCK, {closed: closed, col: this.props.data.col, row: this.props.data.row});
  }
}

Block.defaultProps = {
  description: 'Block'
};