/**
 * @file
 *  日期组件, 依赖 eonasdan-bootstrap-datetimepicker, moment, moment-locales
 * @module widget-calendar.js
 * @version 1.0.0
 */

'use strict';

import ReactDOM from 'react-dom'
import Widget   from './widget'

export default class Calendar extends Widget {

  constructor(props) {
    super(props);
    this.onClick = this.onClick.bind(this);
  }

  componentDidMount() {

    let node = $(ReactDOM.findDOMNode(this)).find('input');

    node.datetimepicker({
      locale: this.locale(),
      format: this.format(),
      defaultDate: this.props.data.defaults,
      ignoreReadonly: true
    });

    if (this.props.data.textMin) {
      node.data('DateTimePicker').minDate(moment(this.props.data.textMin, this.format()));
    }

    if (this.props.data.textMax) {
      node.data('DateTimePicker').maxDate(moment(this.props.data.textMax, this.format()));
    }

    node.on('dp.change', event => {
      this.change(event.date.format(this.format()), event.target.name);
    });
  }

  componentDidUpdate() {

    const node = $(ReactDOM.findDOMNode(this)).find('input'),
      format = this.format();

    if (format !== node.data('DateTimePicker').format()) {
      node.data('DateTimePicker').format(format);
    }
  }

  render() {
    return dom('div', {className: this.className()}, this.icon(), this.widget());
  }

  className() {
    return 'input' + (this.boolean(this.props.data.disabled) ? ' state-disabled' : '');
  }

  getValue() {
    if (this.value()) {
      return moment().format(this.value(), this.format());
    }

    return '';
  }

  format() {
    if (this.props.data.textDateFormat) {
      return this.props.data.textDateFormat;
    }

    if (this.boolean(this.props.data.allowTime)) {
      return 'YYYY-MM-DD HH:mm';
    }

    return 'YYYY-MM-DD';
  }

  locale() {
    return this.props.data.locale || 'zh-cn';
  }

  widget() {
    return dom('input', {
      type: 'text',
      disabled: this.disabled(),
      readOnly: true,
      placeholder: this.props.data.textPlaceholder,
      value: this.getValue(),
      name: this.props.data.name,
      onClick: this.onClick
    });
  }

  onClick() {
    $(ReactDOM.findDOMNode(this)).find('input').data('DateTimePicker').show();
  }
}
