/**
 * @file
 * @module widget-checkbox.js
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'

export default class CheckBox extends Widget {

  constructor(props) {
    super(props);

    this.onChange = this.onChange.bind(this);
  }

  render() {

    this.props.data.selectStyle = this.props.data.selectStyle || this.props.selectStyle;
    this.props.data.option = this.props.data.option || [{value: '01', title: 'item1'}, {value: '02', title: 'item2'}];

    return dom('div', {className: this.className()}, this.widget());
  }

  className() {
    return this.props.data.selectStyle === 'Line' ? 'inline-group' : 'row';
  }

  itemClassName() {
    if (this.props.data.disabled === 'true') {
      return 'checkbox state-disabled';
    }
    return 'checkbox';
  }

  widget() {

    let options = this.props.data.option;

    if (this.props.data.selectStyle === 'Line') {
      return options.map(option => {
        return this.getItem(option);
      });
    }

    if (this.props.data.selectStyle === 'List') {
      return dom('div', {className: 'col-sm-12'},
        options.map(option => {
          return this.getItem(option);
        })
      );
    }

    return [
      dom('div', {className: 'col-sm-6', key: '1'},
        dom('label', {className: this.itemClassName()},
          options.map((option, index) => {
            return index % 2 ? null : this.getItem(option);
          })
        )
      ),
      dom('div', {className: 'col-sm-6', key: '2'},
        dom('label', {className: this.itemClassName()},
          options.map((option, index) => {
            return index % 2 ? this.getItem(option) : null;
          })
        )
      )
    ];
  }

  getItem(option) {

    option = this.normalize(option);

    return dom('label', {className: this.itemClassName(), key: option.value},
      dom('input', {
        type: 'checkbox',
        name: this.props.data.name,
        checked: this.checked(option.value),
        disabled: this.disabled(),
        value: option.value,
        onChange: this.onChange
      }),
      dom('i', ),
      option.title
    );
  }

  checked(option) {
    if (!this.getValue()) {
      return false;
    }

    if (_.isArray(this.getValue())) {
      return this.getValue().includes(option);
    }

    return this.boolean(this.getValue()[option]);
  }

  getValue() {
    if (!this.value() || _.isEmpty(this.value())) {
      return [];
    }

    if (_.isObject(this.value())) {
      return this.value();
    }

    return this.value().split(',');
  }

  /**
   * 支持两种格式的数据存储方式
   *  - hash 选择项目为key 值为true或false
   *  - array 选择项目的数组(数组里包含改选择项目, 则为选中)
   * @param event
   */
  onChange(event) {
    let value = this.getValue();

    // 数组方式保持数据
    if (_.isArray(value)) {
      if (value.includes(event.target.value)) {
        value = _.reject(value, function (val) {
          return val === event.target.value;
        });
      } else {
        value = _.union(value, [event.target.value]);
      }
    }

    // hash方式保持数据
    else {
      value[event.target.value] = !this.boolean(value[event.target.value]);
    }

    this.change(value, event.target.name);
  }
}

CheckBox.defaultProps = {
  selectStyle: 'Line'
};