/**
 * @file
 * @module widget-colortag.js
 * @author
 * @version 1.0.0
 */

'use strict';

import ReactDOM from 'react-dom'
import Widget   from './widget'
import CONST    from '../constant'

export default class ColorTag extends Widget {

  constructor(props) {
    super(props);
    this.state = {tags: this.props.data.tags || this.props.tags, hover: -1, value: ''};

    this.onAddClick = this.onAddClick.bind(this);
    this.onSelectClick = this.onSelectClick.bind(this);
    this.onRemoveClick = this.onRemoveClick.bind(this);
  }

  render() {

    let sorted = _.sortBy(this.getValue(), item => {
      return this.state.tags.indexOf(item);
    });

    return dom('div', {className: 'color-tag'},
      sorted.map(item => {
        return dom('span', {className: 'color-label color-label-front ' + this.color(item), key: item});
      }),
      this.action()
    );
  }

  componentWillReceiveProps(nextProps) {
    if (_.difference(this.state.tags, nextProps.data.tags)) {
      this.setState({tags: nextProps.data.tags || []});
    }
  }

  color(item) {
    return 'bg-color-' + (this.state.tags.indexOf(item) + 1);
  }

  action() {
    return dom('ul', {className: 'action-button nav nav-pills', style: this.props.style},
      dom('li', {className: 'dropup'},
        dom('a', {className: 'dropdown-toggle', 'data-toggle': 'dropdown'},
          dom('i', {className: 'fa fa-fw fa-gear'})
        ),
        dom('ul', {className: 'dropdown-menu'},
          this.state.tags.map((item, index) => {
            return dom('li', {
                key: item,
                'data-id': item,
                onClick: this.onSelectClick,
                onMouseEnter: () => this.setState({hover: index}),
                onMouseLeave: () => this.setState({hover: -1})
              },
              dom('div', {className: this.colorLabelClassName(item, index)},
                dom('p', {
                  className: 'ellipsis',
                  title: item,
                  'data-toggle': 'tooltip',
                  'data-placement': 'bottom'
                }, item),
                dom('div', {className: 'tooltip', style: {top: '24px', left: '5.5px'}}, 'asdfasd'),
                this.removeButton(item, index)
              )
            )
          }),
          dom('li', {className: 'divider'}),
          dom('li', {className: 'nav-item'},
            this.addArea()
          )
        )
      )
    );
  }

  colorLabelClassName(item, index) {
    let selected = this.getValue().indexOf(item) >= 0 ? 'color-label-selected' : '';
    return `color-label ${selected} bg-color-${index + 1}`
  }

  addArea() {
    return dom('div', {className: 'input-group input-group-sm'},
      dom('input', {
        className: 'form-control',
        onChange: event => this.setState({value: event.target.value}),
        value: this.state.value
      }),
      dom('span', {className: 'input-group-btn'},
        dom('button', {className: 'btn btn-default', onClick: this.onAddClick},
          dom('i', {className: 'fa fa-plus'})
        )
      )
    );
  }

  removeButton(item, index) {

    if (this.state.hover == index) {
      return dom('a', {className: 'color-remove', onClick: this.onRemoveClick, 'data-id': item},
        dom('i', {className: 'fa fa-fw fa-close'})
      );
    }
    return null;
  }

  componentDidMount() {
    $(ReactDOM.findDOMNode(this)).tooltip({selector: '[data-toggle=tooltip]', container: 'body'});
  }

  onSelectClick(event) {

    let id = event.currentTarget.getAttribute('data-id')
      , value = this.getValue();

    if (value.indexOf(id) < 0) {
      value.push(id);
    } else {
      value = _.without(value, id);
    }

    this.change(value, this.props.data.name);
  }

  onRemoveClick(event) {
    event.preventDefault();
    event.stopPropagation();

    let id = event.currentTarget.getAttribute('data-id')
      , value = this.getValue();

    this.props.data.tags = _.without(this.props.data.tags, id);

    // 从列表删除
    this.state.tags = _.without(this.state.tags, id);
    this.setState({tags: this.state.tags});

    // 从选择项删除
    value = _.without(value, id);

    this.props.emit(CONST.EVENT.CHANGE, {
      value: value,
      removed: id,
      name: this.props.data.name,
      col: this.props.data.col,
      row: this.props.data.row
    }, ReactDOM.findDOMNode(this));
  }

  onAddClick(event) {
    event.preventDefault();
    event.stopPropagation();

    let added = _.indexOf(this.state.tags, this.state.value) < 0 ? this.state.value : undefined;

    // 添加到列表
    this.state.tags.push(this.state.value);
    this.setState({tags: _.uniq(this.state.tags), value: ''});

    // 添加到选择项
    let value = this.getValue();
    value.push(this.state.value);

    let object = {value: value, name: this.props.data.name, col: this.props.data.col, row: this.props.data.row};
    if (added) {
      object.added = added;
    }

    this.props.emit(CONST.EVENT.CHANGE, object, ReactDOM.findDOMNode(this));
  }

  getValue() {
    if (!this.value()) {
      return [];
    }

    return this.value();
  }
}

ColorTag.defaultProps = {
  tags: []
};
