/**
 * @file
 * @module widget-combobox-dynamic.js
 * @version 1.0.0
 */

'use strict';

import Widget       from './widget'
import MultiSelect  from '../component/combobox'
import * as util    from '../core/util'
import CONST        from '../constant'

export default class ComboBoxDynamic extends Widget {

  constructor(props) {
    super(props);
    this.onChange = this.onChange.bind(this);
    this.onDataChanged = this.onDataChanged.bind(this);

    this.state = {api: this.parseApiParams()};

    // 监听关联项目值变更
    this.props.option.emitter.on(CONST.EVENT.CHANGE, this.onDataChanged);
  }

  onDataChanged(event, data) {

    const name = '@' + data.name
      , value = data.value;

    // 值发生变化的组件并非是关联的项目，则什么也不做
    let params = Object.values(util.params(this.props.data.selectApi));
    if (params.indexOf(name) < 0) {
      return;
    }

    let key = '';

    // 找到发送变化的参数
    params = util.params(this.props.data.selectApi);
    _.each(params, (v, k) => {
      if (name === v) {
        key = k;
      }
    });

    // 替换发送变化的值
    params = util.params(this.state.api);
    params[key] = value === '' ? 'null' : value;

    this.setState({api: this.props.data.selectApi.split('?')[0] + '?' + $.param(params)});
    this.change('', this.props.data.name);
  }

  render() {

    this.props.data.option = this.props.data.option || [];

    return React.createElement(MultiSelect, {
      ref: 'select',
      multiple: this.boolean(this.props.data.selectMultiple) ? 'multiple' : null,
      onChange: this.onChange,
      api: this.state.api,
      dataUk: this.props.data.selectApiUk,
      dataName: this.props.data.selectApiName,
      data: this.props.data.option.map(item => Object.assign({id: item.value, text: item.title})),
      value: this.props.data.value,
      placeholder: this.props.textPlaceholder
    });
  }

  onChange() {
    return this.change(this.refs.select.getValue(), this.props.data.name);
  }

  // 初期显示时，如果关联项目已经有值，那么替换API内的具体参数
  parseApiParams() {

    let api = this.props.data.selectApi;

    Object.values(util.params(api)).forEach(val => {

      // 如果参数里包含@字符开头的值，那么就认为是参数，尝试用form里的项目值替换
      if (val && val[0] === '@') {
        const relate = this.props.option.form.find(item => item.name === val.substr(1));
        if (relate && relate.value) {
          api = api.replace(new RegExp(val, 'g'), relate.value);
        }
      }
    });

    return api;
  }

}

ComboBoxDynamic.defaultProps = {};