/**
 * @file
 * @module widget-file.js
 * @version 1.0.0
 */

'use strict';

import Widget     from './widget'
import FileSelect from '../component/file'

export default class File extends Widget {

  constructor(props) {
    super(props);
    this.onFinish = this.onFinish.bind(this);
  }

  render() {

    const limit = this.props.data.fileLimit || this.props.fileLimit
      , multiple = this.boolean(this.props.data.fileMultiple);

    // 获取值
    const value = this.array(this.props.data.value);

    // FileSelect接收的value与text是数组
    return React.createElement(FileSelect, {
      ref: 'file',
      disabled: this.disabled(),
      value: value.map(item => item.path),
      text: value.map(item => item.name),
      ui: 'selector',
      buttonText: this.props.data.fileButton || this.props.fileButton,
      accept: this.props.data.fileAccept,
      multiple: multiple,
      size: parseInt(limit) * 1024,
      className: {base: 'input-group input-file'},
      success: this.onFinish,
      url: this.props.data.fileUploadApi,
      urltoken: this.props.data.fileTokenApi,
      urldownload: this.props.data.fileApi
    });
  }

  onFinish(result) {
    return this.change(result.map(item => Object.assign({
      path: item._id,
      name: item.filename
    })), this.props.data.name);
  }

}

File.defaultProps = {
  fileLimit: 2048,
  fileButton: ' 选择'
};
