/**
 * @file
 * @module widget-grid.js
 * @version 1.0.0
 */

'use strict';

import ReactDOM from 'react-dom'
import Widget   from './widget'

export default class Grid extends Widget {

  constructor(props) {
    super(props);
  }

  render () {

    if (!this.props.data.gridTitle) {
      this.props.data.gridTitle = ['column1', 'column2'];
    }

    this.props.data.gridBordered = _.isUndefined(this.props.data.gridBordered) ? true : this.props.data.gridBordered;

    return dom('table', {className: this.className(), style: {marginBottom: 0}},
      this.thead(),
      this.tbody()
    );
  }

  className () {

    let bordered = (this.boolean(this.props.data.gridBordered) ? ' table-bordered' : '')
      , striped = (this.boolean(this.props.data.gridStriped) ? ' table-striped' : '');

    return 'table' + bordered + striped;
  }

  thead () {

    let columns = this.props.data.gridTitle.map(function (title) {
      return dom('th', {key: title}, title);
    });

    return dom('thead', null, dom('tr', {
      style: {
        color: this.props.data.gridTitleColor,
        backgroundColor: this.props.data.gridTitleColorBackground
      }
    }, columns));
  }

  getValue () {
    return this.value() || ['', '', '', ''];
  }

  tbody () {

    let rows = [], columns = [], count = this.props.data.gridTitle.length;

    this.getValue().map((value, index)=> {

      columns.push(
        dom('td', {style: {padding: '3px'}},
          this.disabled() ? value : dom('input', {
            type: 'text',
            value: value,
            style: {width: '100%'},
            onChange: this.onChange
          }))
      );

      if ((index + 1) % count == 0 || index == this.getValue().length) {
        rows.push(dom('tr', {key: value + index}, columns));
        columns = [];
      }
    });

    return dom('tbody', null, rows);
  }

  onChange (event) {

    let node = $(ReactDOM.findDOMNode(this)).find('input'), value = _.map(node, function (item) {
      return $(item).val();
    });

    this.change(value, this.props.data.name);
  }
}
