/**
 * @file
 * @module widget-image.js
 * @version 1.0.0
 */

'use strict';

import Widget     from './widget'
import CONST      from '../constant'
import FileSelect from '../component/file'

export default class Image extends Widget {

  constructor(props) {
    super(props);
    this.onClick = this.onClick.bind(this);
    this.onFinish = this.onFinish.bind(this);
  }

  render() {

    // 横向最多显示3张图片
    const width = 12 / this.array(this.props.data.value).length
      , limit = this.props.data.fileLimit || this.props.fileLimit;

    let files = this.array(this.props.data.value);
    files = files.length <= 0 ? [{path: ''}] : files;

    return dom('div', null,
      files.map(file => {
        const src = file.path ? (this.props.data.fileApi + file.path) : CONST.Config.SampleImage;
        return dom('div', {className: `image col-sm-${width < 4 ? 4 : width}`, key: src},
          dom('img', {
            src: src,
            style: {width: this.getWidth(file.path), backgroundColor: file.path ? '#ffffff' : '#c7d0df'}
          }),
          dom('ul', {className: 'image-button'},
            this.renderSelectButton(file.path),
            this.renderRemoveButton(file.path)
          )
        );
      }),
      React.createElement(FileSelect, {
        ref: 'file',
        disabled: this.disabled(),
        value: this.array(this.props.data.value),
        accept: this.props.data.fileAccept || this.props.fileAccept,
        multiple: this.boolean(this.props.data.fileMultiple),
        url: this.props.data.fileUploadApi,
        urltoken: this.props.data.fileTokenApi,
        size: parseInt(limit) * 1024,
        success: this.onFinish
      })
    );
  }

  getWidth(id) {
    if (id) {
      return '100%';
    }

    if (this.props.option.w <= 2) {
      return '40%';
    }

    return ['100%', '100%', '70%', '50%', '40%'][this.props.data.colSpan];
  }

  renderSelectButton(file) {
    if (this.disabled()) {
      return null;
    }

    return dom('li', {},
      dom('i', {
        className: 'fa fa-border fa-plus',
        'data-action': 'select',
        'data-id': file,
        onClick: this.onClick
      }),
      dom('input', {
        style: {display: 'none'},
        type: 'file',
        accept: this.props.data.fileAccept,
        multiple: this.boolean(this.props.data.fileMultiple),
        onChange: this.onChange
      })
    );
  }

  renderRemoveButton(file) {
    if (this.disabled()) {
      return null;
    }

    if (!file) {
      return null;
    }

    return dom('li', {},
      dom('i', {
        className: 'fa fa-border fa-trash',
        'data-action': 'remove',
        'data-id': file,
        onClick: this.onClick
      })
    );
  }

  onClick(event) {

    const action = $(event.target).attr('data-action')
      , id = $(event.target).attr('data-id');

    if (action === 'select') {
      this.current = id;
      return this.refs.file.click();
    }

    if (action === 'remove') {
      if (this.boolean(this.props.data.fileMultiple)) {

        // 删除指定的图片，如果最后文件数组为空那么添加一个''到数组中（渲染组件时，用''来判断是否显示代表空的图片）
        const files = _.reject(this.array(this.props.data.value), item => item.path === id);
        return this.change(files, this.props.data.name);
      }

      this.change([], this.props.data.name);
    }
  }

  onFinish(result) {

    let value = this.array(this.props.data.value);
    if (this.boolean(this.props.data.fileMultiple)) {

      // 其余的照片不动, 只替换选择的图片
      const current = value.find(item => item.path === this.current);
      if (current) {
        current.path = result.shift()._id;
      }

      // 更多的照片补充道最后
      value = value.concat(result.map(item => Object.assign({
        path: item._id,
        width: item.width,
        height: item.height,
        type: 'image'
      })));
      return this.change(value, this.props.data.name)
    }

    this.change([{path: result[0]._id}], this.props.data.name)
  }
}

Image.defaultProps = {
  fileLimit: 2048,
  fileAccept: 'image/png, image/jpeg, image/gif',
  fileApi: CONST.Config.FileImageAPI
};
