/**
 * @file
 * @module widget-number.js
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'

export default class Numerical extends Widget {

  constructor(props) {
    super(props);

    this.onChange = this.onChange.bind(this);
    this.onBlur = this.onBlur.bind(this);
  }

  render() {
    return dom('div', {className: this.className()}, this.icon(), this.widget());
  }

  className() {
    return 'input' + (this.props.data.disabled === 'true' ? ' state-disabled' : '');
  }

  getValue() {
    if (typeof this.value() === 'undefined') {
      return '';
    }

    return this.value();
  }

  widget() {
    return dom('input', {
      type: 'text',
      disabled: this.disabled(),
      placeholder: this.props.data.textPlaceholder,
      onChange: this.onChange,
      onBlur: this.onBlur,
      value: this.getValue(),
      name: this.props.data.name
    });
  }

  onBlur(event) {

    // 允许空值
    if (!event.target.value) {
      return;
    }

    let value = event.target.value.replace(/,/g, '');

    // 转成数字，如果非法数字则换成0
    value = Number(value);
    this.change(_.isNaN(value) ? 0 : value, event.target.name);
  }

  onChange(event) {
    // 数字型，只接受数字
    if (!event.target.value.match(/^[0-9.,\- ]*$/i)) {
      return;
    }
    this.change(event.target.value, event.target.name);
  }
}
