/**
 * @file
 * @module widget-option.js
 * @author
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'

export default class Option extends Widget {

  constructor(props) {
    super(props);

    // Type 是值的类型，默认为String: 支持String，Number，Date
    let defaults = [
      {key: 'value', title: '值', width: '20%', type: 'String'},
      {key: 'title', title: '标题', width: '80%'}
    ];

    this.state = {column: -1, field: this.props.data.field || defaults};
    this.onBlur = this.onBlur.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onFocus = this.onFocus.bind(this);
    this.onAddClick = this.onAddClick.bind(this);
    this.onRemoveClick = this.onRemoveClick.bind(this);
  }

  render() {

    return dom('table', {className: 'option'},
      dom('tbody', {},
        this.getValue().map((data, index) => {
          return this.row(data, index)
        })
      )
    );
  }

  row(data, rowIndex) {

    data = this.normalize(data);

    return dom('tr', {key: rowIndex},
      this.state.field.map((item, colIndex) => {
          return dom('td', {style: this.style(item, colIndex), key: item.key + rowIndex},
            dom('input', {
              type: 'text',
              name: this.props.data.name,
              disabled: this.disabled(),
              placeholder: item.title,
              onFocus: this.onFocus,
              onBlur: this.onBlur,
              onChange: this.onChange,
              'data-key': item.key,
              'data-row': rowIndex,
              'data-column': colIndex,
              'data-type': item.type,
              value: _.isUndefined(data[item.key]) ? '' : data[item.key]
            })
          )
        }
      ),
      dom('td', {}, this.action(rowIndex))
    );
  }

  action(rowIndex) {

    return dom('ul', {className: 'action-button nav nav-pills', style: this.props.style},
      dom('li', {},
        dom('a', {className: 'dropdown-toggle', 'data-toggle': 'dropdown'},
          dom('i', {className: 'fa fa-fw fa-gear'})
        ),
        dom('ul', {className: 'dropdown-menu dropdown-menu-right'},
          dom('li', {onClick: this.onAddClick, name: this.props.data.name, 'data-row': rowIndex},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-plus-circle'}), ' 添加行')
          ),
          dom('li', {className: 'divider'}),
          dom('li', {onClick: this.onRemoveClick, name: this.props.data.name, 'data-row': rowIndex},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-trash-o'}), ' 删除行')
          )
        )
      )
    );
  }

  style(item, column) {
    if (item.width) {
      return {width: item.width};
    }

    if (this.state.column < 0) {
      return {};
    }

    if (column === this.state.column) {
      return {width: '60%'};
    }

    return {width: (40 / (this.state.field.length - 1)) + '%'};
  }

  getValue() {
    return this.value() || [{value: '01'}, {value: '02'}];
  }

  onChange(event) {

    let value = this.getValue()
      , row = event.target.getAttribute('data-row')
      , key = event.target.getAttribute('data-key')
      , type = event.target.getAttribute('data-type');

    if (type === 'Number') {

      // 只接受数字
      if (!event.target.value.match(/^[0-9\/\-: ]*$/i)) {
        return;
      }
    }

    // 进行类型转换
    value[row][key] = event.target.value;
    this.change(value, event.target.name);
  }

  onFocus(event) {
    this.setState({column: event.target.getAttribute('data-column')});
  }

  onBlur() {
    this.setState({column: -1});
  }

  onAddClick(event) {
    event.preventDefault();

    let value = this.getValue()
      , row = parseInt(event.currentTarget.getAttribute('data-row'));

    value.splice(row + 1, 0, {value: ('00' + (row + 2)).substr(-2)});

    this.change(value, event.currentTarget.getAttribute('name'));
  }

  onRemoveClick(event) {
    event.preventDefault();

    let value = this.getValue()
      , row = parseInt(event.currentTarget.getAttribute('data-row'));

    delete value[row];
    this.change(_.compact(value), event.currentTarget.getAttribute('name'));
  }
}

Option.defaultProps = {
  data: {}
};
