/**
 * @file
 * @module widget-plugin.js
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'


export default class Plugin extends Widget {

  constructor(props) {
    super(props);
    this.emit = this.emit.bind(this);
  }

  render() {
    return dom('div', {}, this.component());
  }

  component() {

    // plugin 为全局变量，在插件代码中设定。所以插件的js文件加载需要早与
    if (typeof plugin === 'undefined') {
      return dom('span', {}, 'NO DATE');
    }

    const extend = this.props.data.value || {};
    const items = plugin.filter(item => {

      if (!item.defaultProps) {
        return false;
      }

      if (!item.defaultProps.id) {
        return false;
      }

      return extend.hasOwnProperty(item.defaultProps.id);
    });

    if (items.length <= 0) {
      return dom('span', {}, 'NO DATE');
    }

    return items.map(item => {
      return React.createElement(item, {
        name: this.props.option.name,
        form: this.props.option.form.state.data,
        extend: extend,
        emit: this.emit
      });
    });
  }

  emit(id, val) {
    this.change(Object.assign({}, this.props.data.value, {[id]: val}), this.props.data.name);
  }
}
