/**
 * @file
 * @module widget-radiobox.js
 * @version 1.0.0
 */

'use strict';

import Widget   from './widget'

export default class RadioBox extends Widget {

  constructor(props) {
    super(props);
    this.onChange = this.onChange.bind(this);
  }

  render() {

    this.props.data.selectStyle = this.props.data.selectStyle || this.props.selectStyle;
    this.props.data.option = this.props.data.option || [{value: '01', title: 'item1'}, {value: '02', title: 'item2'}];

    return dom('div', {className: this.className()}, this.widget());
  }

  className() {
    return this.props.data.selectStyle === 'Line' ? 'inline-group' : 'row';
  }

  itemClassName() {
    if (this.props.data.disabled === 'true') {
      return 'radio state-disabled';
    }
    return 'radio';
  }

  widget() {

    let options = this.props.data.option;

    if (this.props.data.selectStyle === 'Line') {
      return options.map(option => {
        return this.getItem(option);
      });
    }

    if (this.props.data.selectStyle === 'List') {
      return dom('div', {className: 'col-sm-12'},
        options.map(option => {
          return this.getItem(option);
        })
      );
    }

    return [
      dom('div', {className: 'col-sm-6', key: '1'},
        dom('label', {className: this.itemClassName()},
          options.map((option, index) => {
            return index % 2 ? null : this.getItem(option);
          })
        )
      ),
      dom('div', {className: 'col-sm-6', key: '2'},
        dom('label', {className: this.itemClassName()},
          options.map((option, index) => {
            return index % 2 ? this.getItem(option) : null;
          })
        )
      )
    ];
  }

  getItem(option) {

    option = this.normalize(option);

    return dom('label', {className: this.itemClassName(), key: option.value},
      dom('input', {
        type: 'radio',
        name: this.props.data.name,
        checked: option.value === this.value(),
        disabled: this.disabled(),
        value: option.value,
        onChange: this.onChange
      }),
      dom('i', {className: 'rounded-x'}),
      option.title
    );
  }

  onChange(event) {
    this.change(event.target.value, event.target.name);
  }
}

RadioBox.defaultProps = {
  selectStyle: 'Line'
};
