/**
 * @file
 * @module widget-select.js
 * @version 1.0.0
 */

'use strict';

import Widget     from './widget'
import SelectBox  from '../component/selectbox'

export default class Select extends Widget {

  constructor(props) {
    super(props);
    this.state = {value: this.value()};
    this.onShowClick = this.onShowClick.bind(this);
    this.onSelect = this.onSelect.bind(this);
  }

  render() {

    let option = this.props.data.option;
    if (!option || option.length < 0) {
      option = [this.props.data.selectApiUk, this.props.data.selectApiName];
    } else {
      option = option.map(item => item.title);
    }

    return dom('div', {},
      React.createElement(SelectBox, {
        ref: 'select',
        api: this.props.data.selectApi,
        dataUk: this.props.data.selectApiUk,
        dataName: this.props.data.selectApiName,
        field: option,
        single: !this.boolean(this.props.data.selectMultiple),
        style: {header: {paddingTop: 15, paddingBottom: 15}, title: {fontSize: 24}},
        ok: this.onSelect,
        value: this.array(this.state.value),
        names: this.array(this.text)
      }),
      dom('div', {className: 'input-group input-select'},
        this.renderButton('left'),
        dom('input', {
          type: 'text',
          readOnly: true,
          style: {backgroundColor: '#fff'},
          value: this.text || ''
        }),
        this.renderButton('right')
      )
    );
  }

  renderButton(side) {
    if (side === this.props.selectButtonPosition) {
      return dom('span', {className: 'input-group-btn', style: {backgroundColor: '#fff'}},
        dom('button', {className: 'btn btn-default', type: 'button', onClick: this.onShowClick},
          dom('i', {className: this.props.data.selectButtonIcon || this.props.selectButtonIcon}),
          dom('span', {}, this.props.data.selectButton || this.props.selectButton)
        )
      );
    }

    return undefined;
  }

  onShowClick() {
    this.refs.select.show();
  }

  onSelect(select) {

    // 显示名称
    this.text = select.getName();

    // 保存选择的ID
    const value = select.getValue();
    if (this.boolean(this.props.data.selectMultiple)) {
      return this.change(value, this.props.data.name);
    }

    this.change(value.length > 0 ? value[0] : '', this.props.data.name);
  }

  componentDidMount() {
    const value = this.array(this.props.data.value);
    if (value.length > 0) {
      this.fetch(value);
    }
  }

  componentDidUpdate() {

    const value = this.array(this.props.data.value);

    if (value.length > 0) {

      // 如果有缺省值，通过网络请求获取名字
      if (!_.isEqual(value, this.refs.select.getValue())) {
        this.fetch(value);
      }
    } else {

      // 空值的时候，清除输入框显示
      if (!_.isEqual(value, this.state.value)) {
        this.text = [];
        this.setState({value: value});
      }
    }
  }

  fetch(value) {
    const condition = {};
    condition[this.refs.select.props.dataKeys] = value;

    this.refs.select.fetch({condition: condition}, result => {
      this.text = result.items.map(item => item[this.refs.select.props.dataName]);
      this.setState({value: value});
    });
  }
}

Select.defaultProps = {
  selectButtonPosition: 'right',
  selectButtonIcon: ' fa fa-plus-square',
  selectButton: ' 选择',
};
