/**
 * @module properties.js
 */

'use strict';

import React    from 'react'
import ReactDOM from 'react-dom'
import CONST    from '../constant'
import Header   from '../form/form-header'
import FieldSet from '../form/form-fieldset'
import Footer   from '../form/form-footer'
import Property from '../form/properties'
import Plugin   from '../form/plugin'
import i        from '../form/localize'


export default class Properties extends React.Component {

  constructor(props) {
    super(props);

    this.emitter = this.props.emitter || $({});

    this.emit = this.emit.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onFold = this.onFold.bind(this);
    this.onPlugin = this.onPlugin.bind(this);

    this.onProcessSelected = this.onProcessSelected.bind(this);
    this.onActivitySelected = this.onActivitySelected.bind(this);
    this.onConditionSelected = this.onConditionSelected.bind(this);

    this.onAddConditionClick = this.onAddConditionClick.bind(this);
    this.onRemoveConditionClick = this.onRemoveConditionClick.bind(this);

    this.state = {
      type: CONST.PROPS_TYPE_PROCESS,                     // 属性框类型
      data: Properties.formatProperties(this.props.data), // 控件 & 数据
      option: {
        plugin: this.props.plugin,                        // 是否显示插件
        mode: CONST.MODE.INPUT,                           // 输入模式
        closed: false,                                    // 是否是关闭状态
        scroll: true,                                     // 固定高度，超过屏幕时显示滚动条
        w: 4,                                             // 列数
        process: this.props.process,
        name: ''                                          // 当前操作的组件name
      },
      button: this.props.button
    };

    // 属性值发生变化，通知表单画面
    this.emitter.on(CONST.EVENT.CHANGE, this.onChange);

    // 折叠属性框
    this.emitter.on(CONST.EVENT.FOLDFORM, this.onFold);

    // 显示插件管理器
    this.emitter.on(CONST.EVENT.PLUGIN, this.onPlugin);

    // 显示属性
    this.props.process.emitter.on(CONST.EVENT.SELECTPROCESS, this.onProcessSelected);
    this.props.process.emitter.on(CONST.EVENT.SELECTACTIVITY, this.onActivitySelected);
    this.props.process.emitter.on(CONST.EVENT.SELECTCONDITION, this.onConditionSelected);
  }


  render() {

    if (this.original) {
      this.state.option.name = this.original.name;
    }

    return dom('form', {className: 'light-form'},
      dom(Header, {
        title: this.title(),
        closed: this.state.option.closed,
        plugin: this.state.option.plugin,
        emit: this.emit
      }),
      this.state.option.plugin
        ? dom(Plugin, {type: this.state.type, data: this.original, emit: this.emit})
        : dom(FieldSet, {data: this.state.data, option: this.state.option, emit: this.emit}),
      dom(Footer, {button: this.state.button})
    );
  }


  componentDidMount() {
    this.setProcessDefaults();
  }


  setProcessDefaults() {

    // 初始化表单的缺省值
    // _.flatten(this.props.properties[CONST.PROPS_TYPE_PROCESS]).forEach(item => {
    //   if (_.isUndefined(item.defaults)) {
    //     return;
    //   }
    //   if (_.isUndefined(this.props.form.meta[item.name])) {
    //     this.props.form.meta[item.name] = item.defaults;
    //   }
    // });
  }


  title() {
    if (this.state.type === CONST.PROPS_TYPE_PROCESS) {
      return i('workflow.process.property');
    }

    if (this.state.type === CONST.PROPS_TYPE_ACTIVITY) {
      return i('workflow.activity.property');
    }

    if (this.state.type === CONST.PROPS_TYPE_BRANCH) {
      return i('workflow.condition.property');
    }

    return '';
  }


  /**
   * 发射事件（当前属性表单内有效，防止一个页面有多个Form时，事件互相影响）
   */
  emit(type, data, node) {
    this.emitter.trigger(type, [data, node]);
  }


  /**
   * 属性头的折叠事件
   */
  onFold(event, state) {
    this.state.option.closed = state;
    this.setState({option: this.state.option});
  }


  /**
   * 显示插件一览
   */
  onPlugin(event, state) {
    this.state.option.plugin = state;
    this.setState({option: this.state.option});
  }


  /**
   * 属性值发生了变化
   * @param event
   * @param data
   */
  onChange(event, data) {

    // 流程属性
    if (this.state.type === CONST.PROPS_TYPE_PROCESS) {

      this.original[data.name] = data.value;
      this.props.process.setProcessInfo({}, {[data.name]: data.value});

      this.props.onChange(data, this.state.data, this.state.type);
      return this.onProcessSelected();
    }

    // 节点属性
    if (this.state.type === CONST.PROPS_TYPE_ACTIVITY) {

      this.original[data.name] = data.value;
      this.props.process.setActivityInfo(this.index, {[data.name]: data.value});

      this.props.onChange(data, this.state.data, this.state.type);
      return this.onActivitySelected(null, this.original, this.index);
    }

    // 条件属性
    if (this.state.type === CONST.PROPS_TYPE_BRANCH) {

      this.original[data.name] = data.value;
      this.props.process.setBranchInfo(this.index, {[data.name]: data.value});

      this.props.onChange(data, this.state.data, this.state.type);
      return this.onConditionSelected(null, this.original, this.index);
    }
  }

  // 流程属性
  onProcessSelected() {

    this.original = this.props.process.props.meta;

    const props = this.props.properties[CONST.PROPS_TYPE_PROCESS];

    this.state.data = Properties.formatProperties(props, this.original);
    this.state.data = this.props.onProcessWillShow(this.state.data, CONST.PROPS_TYPE_PROCESS);
    this.setState({type: CONST.PROPS_TYPE_PROCESS, data: this.state.data});
  }

  // 节点属性
  onActivitySelected(event, activity, index) {

    this.original = activity;
    this.index = index;

    const props = this.props.properties[CONST.PROPS_TYPE_ACTIVITY];

    // 恢复操作按钮
    this.state.button = this.props.button;

    let data = _.cloneDeep(Properties.formatProperties(props, this.original));

    // 判断节点类型是开始或者结束或者动态
    const typeItem = data.find(item => item.name === CONST.ACTIVITY_TYPE);
    if (activity.type === CONST.ACTIVITY_TYPE_START) {
      typeItem.disabled = true;
      data = data.filter(item => ['type', 'name', 'id'].includes(item.name));
    }

    // 结束节点时，只留类型与名称栏
    if (activity.type === CONST.ACTIVITY_TYPE_END) {
      typeItem.disabled = true;
      data = data.filter(item => ['type', 'name', 'id'].includes(item.name));
    }

    // 动态节点时，只留类型栏
    if (typeItem.value === CONST.ACTIVITY_TYPE_DYNAMIC) {
      data = data.filter(item => ['type', 'name', 'id'].includes(item.name));
      typeItem.option = typeItem.option.filter(
        item => item.value === CONST.ACTIVITY_TYPE_DYNAMIC || item.value === CONST.ACTIVITY_TYPE_STATIC
      );
    }

    if (typeItem.value === CONST.ACTIVITY_TYPE_STATIC) {
      typeItem.option = typeItem.option.filter(
        item => item.value === CONST.ACTIVITY_TYPE_DYNAMIC || item.value === CONST.ACTIVITY_TYPE_STATIC
      );
    }

    data = this.props.onActivityWillShow(data, CONST.PROPS_TYPE_ACTIVITY);
    this.setState({type: CONST.PROPS_TYPE_ACTIVITY, data: data, button: this.state.button});
  }

  // 分支属性
  onConditionSelected(event, branch, index) {

    this.original = branch;
    this.index = index;

    // 设定操作按钮
    this.state.button = [Properties.createButton('添加条件', this.onAddConditionClick)];

    // 如果没有条件内容，清除所有属性，并返回
    if (!this.original) {
      this.state.data = [];
      return this.setState({type: CONST.PROPS_TYPE_BRANCH, data: this.state.data, button: this.state.button});
    }

    this.state.button.push(Properties.createButton('删除条件', this.onRemoveConditionClick));

    // 尝试调用用户自定义方法
    const props = this.createConditionProperties(this.original.length);

    // 将值组织成 {key_index: value} 格式的对象
    let original = {};
    this.original.forEach((item, index) => {
      Object.keys(item).forEach(key => original[key + '_' + index] = item[key]);
    });
    this.state.data = Properties.formatProperties(props, original);

    this.state.data = this.props.onConditionWillShow(this.state.data, CONST.PROPS_TYPE_BRANCH);
    this.setState({type: CONST.PROPS_TYPE_BRANCH, data: this.state.data, button: this.state.button});
  }

  // 根据条件的个数，生成条件属性
  createConditionProperties(length) {

    let props = [];

    for (let i = 0; i < length; i++) {
      props = props.concat(_.cloneDeep(this.props.properties[CONST.PROPS_TYPE_BRANCH]).map(item => {
        if (Array.isArray(item)) {
          item.forEach(ite => ite.name = `${ite.name}_${i}`);
        } else {
          item.name = `${item.name}_${i}`;
        }
        return item;
      }));
    }

    props.pop();
    return props;
  }

  onAddConditionClick() {
    this.props.process.emit(CONST.EVENT.ADDCONDITION, true);
  }

  onRemoveConditionClick() {
    this.props.process.emit(CONST.EVENT.REMOVECONDITION, true);
  }

  static createButton(name, event) {
    return {
      name: name, className: 'btn btn-primary btn-sm pull-left', onClick: event
    };
  }

  static formatProperties(data = [], original = {}) {

    let result = [];

    data.forEach((item, row) => {

      if (!Array.isArray(item)) {
        item = [item];
      }

      item.forEach((sub, col) => {

        // 多国语言对应
        Property.translate(sub);

        // 配置文件里，为了方便没有填写组件所在的行数及宽带，这里根据顺序计算row, col等信息
        sub.row = row;
        sub.col = col;
        sub.rowSpan = 1;
        sub.colSpan = 4 / item.length;

        // 设定取值 (或缺省值)
        sub.value = (typeof original[sub.name] === 'undefined') ? sub.defaults : original[sub.name];
        result.push(sub);
      });
    });

    return result;
  }

}


Properties.defaultProps = {
  emitter: undefined,
  properties: {},
  process: {},
  data: [],
  button: [],
  onChange: (source, widgets, type) => {
  },
  onProcessWillShow: (widgets, type) => {
    return widgets;
  },
  onActivityWillShow: (widgets, type) => {
    return widgets;
  },
  onConditionWillShow: (widgets, type) => {
    return widgets;
  }
};


Properties.init = (node, option) => {
  return ReactDOM.render(dom(Properties, option), document.getElementById(node));
};
