/**
 * workflow-activity.js
 */

'use strict';

import CONST      from '../constant'
import i          from '../form/localize'
import Properties from './properties'

export default class Activity extends React.Component {

  constructor(props) {
    super(props);

    this.state = {
      height: null
    };

    this.onConditionClick = this.onConditionClick.bind(this);
    this.onActivityClick = this.onActivityClick.bind(this);
    this.onAddActivityClick = this.onAddActivityClick.bind(this);
    this.onAddSubActivityClick = this.onAddSubActivityClick.bind(this);
    this.onAddConditionClick = this.onAddConditionClick.bind(this);
    this.onRemoveConditionClick = this.onRemoveConditionClick.bind(this);
    this.onRemoveActivityClick = this.onRemoveActivityClick.bind(this);
  }

  render() {
    return dom('fieldset', {
        className: 'process-container',
        style: {height: this.state.height, maxHeight: this.props.style.height}
      },
      dom('ul', {className: 'process'},
        this.renderItem(this.props.activities)
      )
    );
  }

  componentDidMount() {

    if (!this.props.style.scroll) {
      return;
    }

    const node = ReactDOM.findDOMNode(this), offset = this.props.style.offset || 240;

    $(window).resize(() => {
      this.setState({height: (window.innerHeight - node.offsetTop - offset) + 'px'});
    });

    this.setState({height: (window.innerHeight - node.offsetTop - offset) + 'px'});
  }

  renderItem(activities, child = false) {

    return activities.map((activity, index) => {

      const isActive = activity.id === this.props.selected.id && !this.props.selected.transition
        , APPROVE_TYPE = {'AND': '会签', 'XOR': '或签'};

      return dom('li', {className: 'process-step'},
        dom('div', {className: 'left-blank'}, dom('span', null, child ? '' : 'Step ' + (index + 1))),
        dom('i', {className: 'activity-icon'}),
        dom('div', {
            className: `activity ${isActive ? 'activity-active' : ''}`,
            onClick: this.onActivityClick,
            aid: activity.id
          },
          this.renderAction(activity),
          dom('p', {className: 'ellipsis'}, activity.name || activity.id),
          dom('div', null,
            dom('span', {className: 'description'}, activity.description || ''),
            activity.performerType ?
              dom('span', {className: 'label label-tag'},
                dom('i', {className: 'fa fa-tags'}),
                ' ' + this.getPropName('Activity', 'performerType', activity.performerType)
              ) : null,
            activity.approveType ?
              dom('span', {className: 'label label-tag'},
                dom('i', {className: 'fa fa-tags'}), ' ' + APPROVE_TYPE[activity.approveType]
              ) : null
          )
        ),

        this.renderSubItem(activity)
      );
    });
  }

  renderAction(activity) {

    if (this.props.mode === CONST.MODE.INPUT) {
      return;
    }

    if (activity.type === CONST.ACTIVITY_TYPE_END) {
      return;
    }

    if (activity.id !== this.props.selected.id || this.props.selected.transition) {
      return null;
    }

    return dom('ul', {className: `action-button-right nav nav-pills`},
      dom('li', {},
        dom('a', {className: 'dropdown-toggle', 'data-toggle': 'dropdown'},
          dom('i', {className: 'fa fa-fw fa-pencil-square-o'})
        ),
        dom('ul', {className: 'dropdown-menu dropdown-menu-right'},
          dom('li', {aid: activity.id, onClick: this.onAddActivityClick},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-plus-circle'}), ' ' + i('添加步骤'))
          ),
          dom('li', {aid: activity.id, onClick: this.onAddSubActivityClick},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-plus-circle'}), ' ' + i('添加分支步骤'))
          ),
          activity.type === CONST.ACTIVITY_TYPE_START ? null :
            dom('li', {className: 'divider'}),
          activity.type === CONST.ACTIVITY_TYPE_START ? null :
            dom('li', {onClick: this.onRemoveActivityClick},
              dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-trash'}), '' + i('删除'))
            )
        )
      )
    );
  }

  renderConditionAction(id, transition, condition) {

    if (this.props.mode === CONST.MODE.INPUT) {
      return;
    }

    const isActive = id === this.props.selected.id
      && transition.id === this.props.selected.transition
      && condition === this.props.selected.condition;

    if (!isActive) {
      return;
    }

    return dom('ul', {className: 'nav nav-pills action-button-right small-action-button-right'},
      dom('li', {},
        dom('a', {className: 'dropdown-toggle', 'data-toggle': 'dropdown'},
          dom('i', {className: 'fa fa-fw fa-pencil-square-o'})
        ),
        dom('ul', {className: 'dropdown-menu dropdown-menu-right'},
          dom('li', {aid: id, onClick: this.onAddConditionClick},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-plus-circle'}), ' ' + i('添加条件组'))
          ),
          transition.condition.length > 0 &&
          dom('li', {className: 'divider'}),
          transition.condition.length > 0 &&
          dom('li', {aid: id, onClick: this.onRemoveConditionClick},
            dom('a', {href: '#'}, dom('i', {className: 'fa fa-fw fa-trash'}), ' ' + i('删除条件组'))
          )
        )
      )
    );
  }

  // 渲染子项目
  renderSubItem(activity) {

    if (this.props.mode === CONST.MODE.INPUT) {
      return;
    }

    if (!activity.children || activity.children.length <= 0) {
      return null;
    }

    return [
      dom('div', {className: 'condition-line'}),
      dom('div', {className: 'row condition-block'},

        activity.children.map(child => {
            return dom('div', {className: 'col-sm-' + (12 / activity.children.length)},
              this.renderCondition(activity.id, child),
              dom('ul', {className: 'subprocess'},
                this.renderItem(child.activities, true)
              )
            );
          }
        )
      )];
  }

  // 渲染条件
  renderCondition(id, activity) {

    const transition = activity.transitions;
    if (!transition || !transition.condition) {
      return null;
    }

    if (transition.condition.length) {
      return dom('div', {className: 'row condition'},
        transition.condition.map((item, i) => {
            return dom('div', {
                className: `col-lg-${12 / transition.condition.length} item`,
                aid: id,
                tid: transition.id,
                cid: i,
                onClick: this.onConditionClick
              },
              this.renderConditionAction(id, transition, i),
              dom('p', {className: 'ellipsis'}, this.getConditionName(transition, item, i))
            )
          }
        )
      );
    } else {
      return dom('div', {className: 'row condition'},
        dom('div', {className: `col-lg-12 item`, aid: id, tid: transition.id, cid: 0, onClick: this.onConditionClick},
          this.renderConditionAction(id, transition, 0),
          dom('p', {className: 'ellipsis'}, '无条件')
        )
      );
    }
  }

  // 获取条件的描述
  getConditionName(transition, item, i) {

    let condition = item.map(and => {
      if (!and.key || typeof and.value === 'undefined' || and.value === '') {
        return '';
      }
      return and.key + ' '
        + this.getPropName('Branch', 'operator', and.operator) + ' '
        + and.value
    });

    condition = condition.filter(item => item);
    if (condition.length <= 0) {
      return '条件未设定';
    }

    const or = (transition.condition.length > 1 && i > 0) ? '或' : '';
    return or + condition.join(' 且 ');
  }

  // 获取分支条件value对应的title
  getPropName(block, name, value) {
    let props = Properties.defaultProps.properties[block].find(item => {
      return Array.isArray(item) ? item.find(it => it.name === name) : item.name === name;
    });

    if (Array.isArray(props)) {
      props = props.find(item => item.name === name);
    }

    return (props.option.find(item => item.value === value) || {}).title;
  }

  onAddActivityClick(event) {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.ADDACTIVITY, event.currentTarget.getAttribute('aid'));
  }

  onAddSubActivityClick(event) {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.ADDSUBACTIVITY, event.currentTarget.getAttribute('aid'));
  }

  onAddConditionClick() {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.ADDCONDITION);
  }

  onRemoveConditionClick(event) {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.REMOVECONDITION);
  }

  onRemoveActivityClick(event) {

    event.preventDefault();
    event.stopPropagation();

    this.props.emit(CONST.EVENT.REMOVEACTIVITY);
  }

  onActivityClick(event) {

    const aid = event.currentTarget.getAttribute('aid')
      , activity = Activity.toList(this.props.activities).find(item => item.id === aid);

    this.props.emit(CONST.EVENT.SELECTACTIVITY, activity, {aid: aid});
  }

  onConditionClick(event) {

    const aid = event.currentTarget.getAttribute('aid')
      , activity = Activity.toList(this.props.activities).find(item => item.id === aid);

    const tid = event.currentTarget.getAttribute('tid')
      , transition = activity.transitions.find(item => item.id === tid);

    const cid = parseInt(event.currentTarget.getAttribute('cid'));
    this.props.emit(CONST.EVENT.SELECTCONDITION, transition.condition[cid], {aid: aid, tid: tid, cid: cid});
  }

  /**
   * 将列表的activity转换成树状结构
   * @param list
   * @param tree
   * @param root
   */
  static toTree(list, tree = [], root) {

    // 获取入口 activity
    if (!root) {
      root = list.find(item => item.type === CONST.ACTIVITY_TYPE_START);
    }

    const alreadyExits = tree.find(item => item.id === root.id);
    if (!alreadyExits) {
      tree.push(root);
    }

    root.children = [];
    root.transitions.forEach(transition => {

      // 找到 to 指向的 activity
      const activity = list.find(item => item.id === transition.to);

      // 如果 condition 存在则缩进。condition 是空数组则是分支，否则是条件分支。
      if (transition.condition) {

        // 构建children项目
        const child = {transitions: transition, activities: []};
        root.children.push(child);

        Activity.toTree(list, child.activities, activity);
      } else {

        // to 为空时（没有后续的Activity）
        if (activity) {
          Activity.toTree(list, tree, activity);
        }
      }
    });

    return tree;
  }

  /**
   * 将树状结构的activity转换成列表
   * @param tree
   * @param list
   * @returns {Array}
   */
  static toList(tree, list = []) {

    tree.forEach(item => {
      list.push(item);
      if (item.children) {
        item.children.forEach(children => {
          Activity.toList(children.activities, list);
        });
      }
    });

    return list;
  }


  /**
   * 设定Route信息
   * @param list
   */
  static resetRoute(list) {
    list.forEach(item => {
      if (item.to) {
        delete item.route;
      }
    });
  }
}

Activity.defaultProps = {
  style: {}
};
